/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    :   adslAtucPerfDataTable.c
 * Author       :   Subramani
 * Date         :   17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      :   Amazon
 * Block        :   ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *               1) Initializing the adslAtucPerfDataTable module
 *               2) Handling requests for the adslAtucPerfDataTable table
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */



/* ===========================================================================
 * Revision History:
 * Subramani:Data type of adslAtucPerfValidIntervals ,
 * adslAtucPerfInvalidIntervals and   adslAtucPerfPrev1DayMoniSecs changed
 * from long to int
 * $Log$
 * ===========================================================================
 */

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.iterate_access.conf,v 1.6 2003/09/29 22:30:36 hardaker Exp $
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
*/

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "adslAtucPerfDataTable.h"
#include "adslAtucPerfDataTable_access.h"

/*
 * ===========================================================================
 *                          LOCAL FUNCTIONS
 * ===========================================================================
*/

/** Initialize the adslAtucPerfDataTable table by defining its contents
    and how it's structured */
void
initialize_table_adslAtucPerfDataTable(void)
{
    static oid      adslAtucPerfDataTable_oid[] =
        { 1, 3, 6, 1, 2, 1, 10, 94, 1, 1, 6 };
    netsnmp_table_registration_info *table_info;
    netsnmp_handler_registration *my_handler;
    netsnmp_iterator_info *iinfo;

    /** create the table registration information structures */
    table_info = SNMP_MALLOC_TYPEDEF(netsnmp_table_registration_info);
    iinfo = SNMP_MALLOC_TYPEDEF(netsnmp_iterator_info);

    /** if your table is read only, it's easiest to change the
        HANDLER_CAN_RWRITE definition below to HANDLER_CAN_RONLY */
    my_handler =
        netsnmp_create_handler_registration("adslAtucPerfDataTable",
                                            adslAtucPerfDataTable_handler,
                                            adslAtucPerfDataTable_oid,
                                            OID_LENGTH
                                            (adslAtucPerfDataTable_oid),
                                            HANDLER_CAN_RONLY);

    if (!my_handler || !table_info || !iinfo) {

#ifdef AMAZON_MEM_LEAK_FIX
         if (my_handler)
            SNMP_FREE(my_handler);
         if (table_info)
            SNMP_FREE(table_info);
          if (iinfo)
            SNMP_FREE(iinfo);
#endif
        snmp_log(LOG_ERR,\
                 "malloc failed in initialize_table_adslAtucPerfDataTable");
        return; /** Serious error. */
    }

    /***************************************************
     * Setting up the table's definition
     */
    netsnmp_table_helper_add_indexes(table_info, ASN_INTEGER,
                                               /** index: ifIndex */
                                     0);

    /** Define the minimum and maximum accessible columns.  This
        optimizes retrival. */
    table_info->min_column = 1;
    table_info->max_column = 29;

    /** iterator access routines */
    iinfo->get_first_data_point =
        adslAtucPerfDataTable_get_first_data_point;
    iinfo->get_next_data_point = adslAtucPerfDataTable_get_next_data_point;

    /** you may wish to set these as well */
#ifdef MAYBE_USE_THESE
    iinfo->make_data_context =
        adslAtucPerfDataTable_context_convert_function;
    iinfo->free_data_context = adslAtucPerfDataTable_data_free;

    /** pick *only* one of these if you use them */
    iinfo->free_loop_context = adslAtucPerfDataTable_loop_free;
    iinfo->free_loop_context_at_end = adslAtucPerfDataTable_loop_free;
#endif

    /** tie the two structures together */
    iinfo->table_reginfo = table_info;

    /***************************************************
     * registering the table with the master agent
     */
    DEBUGMSGTL(("initialize_table_adslAtucPerfDataTable",\
              "Registering table adslAtucPerfDataTable as a table iterator\n"));
    netsnmp_register_table_iterator(my_handler, iinfo);
}

/** Initializes the adslAtucPerfDataTable module */
void
init_adslAtucPerfDataTable(void)
{

  /** here we initialize all the tables we're planning on supporting */
    initialize_table_adslAtucPerfDataTable();
}

/** handles requests for the adslAtucPerfDataTable table, if anything else
    needs to be done */
int
adslAtucPerfDataTable_handler(netsnmp_mib_handler *handler,
                              netsnmp_handler_registration *reginfo,
                              netsnmp_agent_request_info *reqinfo,
                              netsnmp_request_info *requests)
{

    netsnmp_request_info *request;
    netsnmp_table_request_info *table_info;
    netsnmp_variable_list *var;
    struct commitInfo *ci = NULL;

    void           *data_context = NULL;

#ifdef ENABLE_DRIVER_CALL
    Fd = dev_open(ADSL_DEV_PATH, O_RDONLY);
    if (Fd <=0) {
        snmp_log (LOG_ERR, "Unable to open the device... exiting!\n");
        return SNMP_ERR_GENERR;
    }
#endif

    for (request = requests; request; request = request->next) {
        var = request->requestvb;
        if (request->processed != 0)
            continue;

        switch (reqinfo->mode) {
        case MODE_GET:

            data_context = netsnmp_extract_iterator_context(request);
            if (data_context == NULL) {
                if (reqinfo->mode == MODE_GET) {
                    netsnmp_set_request_error(reqinfo, request,
                                              SNMP_NOSUCHINSTANCE);
                    continue;
                }
            }

        }

        /** extracts the information about the table from the request */
        table_info = netsnmp_extract_table_info(request);
        /** table_info->colnum contains the column number requested */
        /** table_info->indexes contains a linked list of snmp variable
           bindings for the indexes of the table.  Values in the list
           have been set corresponding to the indexes of the
           request */
        if (table_info == NULL) {
            continue;
        }

        switch (reqinfo->mode) {
        case MODE_GET:
            switch (table_info->colnum) {
#if defined(HAVE_OPTIMIZED_CODE)
	    case COLUMN_ADSLATUCPERFLOFS:
	    case COLUMN_ADSLATUCPERFLOSS:
	    case COLUMN_ADSLATUCPERFESS:
	    case COLUMN_ADSLATUCPERFINITS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfDataLoss(data_context, &retval_len,
					table_info->colnum);
                    snmp_set_var_typed_value(var, ASN_COUNTER,
                                       (const u_char *) retval, retval_len);
                }
		break;

	    case COLUMN_ADSLATUCPERFVALIDINTERVALS:
	    case COLUMN_ADSLATUCPERFINVALIDINTERVALS:
	    case COLUMN_ADSLATUCPERFPREV1DAYMONISECS:
                {
                    int           *retval;
                    size_t         retval_len = 0;
                    retval =
                        get_adslAtucPerfIntervalData(data_context, &retval_len,
					table_info->colnum);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                       (const u_char *) retval, retval_len);
                }
		break;

	    case COLUMN_ADSLATUCPERFCURR15MINTIMEELAPSED:
	    case COLUMN_ADSLATUCPERFCURR15MINLOFS:
	    case COLUMN_ADSLATUCPERFCURR15MINLOSS:
	    case COLUMN_ADSLATUCPERFCURR15MINESS:
	    case COLUMN_ADSLATUCPERFCURR15MININITS:
	    case COLUMN_ADSLATUCPERFCURR1DAYTIMEELAPSED:
	    case COLUMN_ADSLATUCPERFCURR1DAYLOFS:
	    case COLUMN_ADSLATUCPERFCURR1DAYLOSS:
	    case COLUMN_ADSLATUCPERFCURR1DAYESS:
	    case COLUMN_ADSLATUCPERFCURR1DAYINITS:
	    case COLUMN_ADSLATUCPERFPREV1DAYLOFS:
	    case COLUMN_ADSLATUCPERFPREV1DAYLOSS:
	    case COLUMN_ADSLATUCPERFPREV1DAYESS:
	    case COLUMN_ADSLATUCPERFPREV1DAYINITS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfDataStats(data_context, &retval_len,
					table_info->colnum);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                       (const u_char *) retval, retval_len);
                }
                break;

#else
            case COLUMN_ADSLATUCPERFLOFS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfLofs(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_COUNTER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFLOSS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfLoss(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_COUNTER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFESS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfESs(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_COUNTER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFINITS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfInits(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_COUNTER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFVALIDINTERVALS:
                {
                      int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfValidIntervals(data_context,
                                                       &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFINVALIDINTERVALS:
                {
                    int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfInvalidIntervals(data_context,
                                                         &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFCURR15MINTIMEELAPSED:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfCurr15MinTimeElapsed(data_context,
                                                             &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFCURR15MINLOFS:
                {
                    u_int32          *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfCurr15MinLofs(data_context,
                                                      &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                            (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFCURR15MINLOSS:
                {
                    u_int32          *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfCurr15MinLoss(data_context,
                                                      &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                            (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFCURR15MINESS:
                {
                    u_int32          *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfCurr15MinESs(data_context,
                                                     &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFCURR15MININITS:
                {
                    u_int32          *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfCurr15MinInits(data_context,
                                                       &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                            (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFCURR1DAYTIMEELAPSED:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfCurr1DayTimeElapsed(data_context,
                                                            &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFCURR1DAYLOFS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfCurr1DayLofs(data_context,
                                                     &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFCURR1DAYLOSS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfCurr1DayLoss(data_context,
                                                     &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

             case COLUMN_ADSLATUCPERFCURR1DAYESS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfCurr1DayESs(data_context,
                                                    &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFCURR1DAYINITS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfCurr1DayInits(data_context,
                                                      &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFPREV1DAYMONISECS:
                {
                     int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfPrev1DayMoniSecs(data_context,
                                                         &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFPREV1DAYLOFS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfPrev1DayLofs(data_context,
                                                     &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFPREV1DAYLOSS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfPrev1DayLoss(data_context,
                                                     &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFPREV1DAYESS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfPrev1DayESs(data_context,
                                                    &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCPERFPREV1DAYINITS:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucPerfPrev1DayInits(data_context,
                                                      &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;
#endif
	    default:
                /** We shouldn't get here */
                snmp_log(LOG_ERR,\
                         "adslAtucPerfDataTable_handler: unknown column\n");
            }
            break;

        default:
            snmp_log(LOG_ERR,\
                     "adslAtucPerfDataTable_handler: unsupported mode\n");
        }
    }

#ifdef ENABLE_DRIVER_CALL
    dev_close(Fd);
#endif

    return SNMP_ERR_NOERROR;
}
